/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */

package com.kakao.sdk.partner.user.network

import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ApiErrorCause
import com.kakao.sdk.common.model.ApiErrorResponse
import com.kakao.sdk.common.model.ApplicationContextInfo
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.partner.auth.verifyAge
import okhttp3.Interceptor
import okhttp3.Response
import okhttp3.ResponseBody
import java.util.concurrent.CountDownLatch
/**
 * @suppress
 */
class RequiredAgeVerificationInterceptor(
    private val contextInfo: ApplicationContextInfo = KakaoSdk.applicationContextInfo)
    : Interceptor {
    override fun intercept(chain: Interceptor.Chain): Response {
        val originalResponse = chain.proceed(chain.request())

        if (originalResponse.isSuccessful) {
            return originalResponse
        }

        val body = originalResponse.body()
        val bodyString = body?.string()

        val response = originalResponse.newBuilder()
            .body(ResponseBody.create(body?.contentType(), bodyString))
            .build()

        val apiErrorResponse = bodyString?.let {
            KakaoJson.fromJson<ApiErrorResponse>(it, ApiErrorResponse::class.java)
        }
        val apiErrorCause = apiErrorResponse?.let {
            KakaoJson.fromJson<ApiErrorCause>(it.code.toString(), ApiErrorCause::class.java)
        }

        // 405 연령인증 필요
        if (apiErrorCause == ApiErrorCause.RequiredAgeVerification) {
            var error: Throwable? = null
            val latch = CountDownLatch(1)

            AuthApiClient.instance.verifyAge(context = contextInfo.applicationContext) { e ->
                error = e; latch.countDown()
            }

            latch.await()
            error?.let {
                throw java.io.IOException(error)
            } ?:  return chain.proceed(response.request())
        }
        return response
    }
}
